package goamz

import (
    "http"
    "os"
    "sort"
    "strconv"
    "time"
    "xml"
)

type EC2 struct {
    Auth
    Region
    private byte
}

func NewEC2(auth Auth, region Region) *EC2 {
    return &EC2{auth, region, 0}
}


// ----------------------------------------------------------------------------
// Filtering helper.

type Filter struct {
    m map[string][]string
}

func NewFilter() *Filter {
    return &Filter{make(map[string][]string)}
}

func (f *Filter) Add(name string, value ...string) {
    f.m[name] = append(f.m[name], value...)
}

func (f *Filter) addParams(params map[string]string) {
    if f != nil {
        a := make(sort.StringArray, len(f.m))
        i := 0
        for k := range f.m {
            a[i] = k
            i++
        }
        a.Sort()
        for i, k := range a {
            prefix := "Filter." + strconv.Itoa(i+1)
            params[prefix+".Name"] = k
            for j, v := range f.m[k] {
                params[prefix+".Value."+strconv.Itoa(j+1)] = v
            }
        }
    }
}

// ----------------------------------------------------------------------------
// Request dispatching logic.

type EC2Error struct {
    StatusCode int
    Code       string
    Message    string
    RequestId  string
}

func (err *EC2Error) String() string {
    return err.Message
}

// For now a single error instance is being exposed. In the future it may be useful
// to provide access to all of them, but rather than doing it as an array/slice,
// use a *next pointer, so that it's backward compatible and it continues to be
// easy to handle the first error, which is what most people will want.
type xmlEC2Errors struct {
    RequestId string
    Errors    []EC2Error "Errors>Error"
}

func (ec2 *EC2) query(params map[string]string, resp interface{}) os.Error {
    params["Version"] = "2010-11-15"
    params["Timestamp"] = time.UTC().Format(time.RFC3339)
    url, err := http.ParseURL(ec2.Region.EC2Endpoint)
    if err != nil {
        return err
    }
    signEC2(ec2.Auth, "GET", "/", params, url.Host)
    url.RawQuery = http.EncodeQuery(multimap(params))
    r, _, err := http.Get(url.String())
    if err != nil {
        return err
    }
    //dump, _ := http.DumpResponse(r, true)
    //println("DUMP:\n", string(dump))
    if r.StatusCode != 200 {
        return buildError(r)
    }
    err = xml.Unmarshal(r.Body, resp)
    r.Body.Close()
    return err
}

func multimap(p map[string]string) map[string][]string {
    q := make(map[string][]string, len(p))
    for k, v := range p {
        q[k] = []string{v}
    }
    return q
}

func buildError(r *http.Response) os.Error {
    errors := xmlEC2Errors{}
    xml.Unmarshal(r.Body, &errors)
    r.Body.Close()
    var err EC2Error
    if len(errors.Errors) > 0 {
        err = errors.Errors[0]
    }
    err.RequestId = errors.RequestId
    err.StatusCode = r.StatusCode
    if err.Message == "" {
        err.Message = r.Status
    }
    return &err
}

func makeParams(action string) map[string]string {
    params := make(map[string]string)
    params["Action"] = action
    return params
}

// ----------------------------------------------------------------------------
// RunInstances (http://goo.gl/Mcm3b)

type RunInstances struct {
    ImageId               string
    MinCount              int
    MaxCount              int
    InstanceType          string
    SecurityGroups        []string
    KernelId              string
    RamdiskId             string
    UserData              []byte
    AvailZone             string
    GroupName             string
    Monitoring            bool
    SubnetId              string
    DisableAPITermination bool
    ShutdownBehavior      string
    PrivateIPAddress      string
}

type RunInstancesResp struct {
    RequestId      string
    ReservationId  string
    OwnerId        string
    SecurityGroups []string   "GroupSet>Item>GroupId"
    Instances      []Instance "InstancesSet>Item"
}

type Instance struct {
    InstanceId     string
    InstanceType   string
    ImageId        string
    PrivateDNSName string
    DNSName        string
    KeyName        string
    AMILaunchIndex int
    Hypervisor     string
    VirtType       string "VirtualizationType>"
    Monitoring     string "Monitoring>State"
    AvailZone      string "Placement>AvailabilityZone"
    GroupName      string "Placement>GroupName"
}

func (ec2 *EC2) RunInstances(options *RunInstances) (resp *RunInstancesResp, err os.Error) {
    params := makeParams("RunInstances")
    params["ImageId"] = options.ImageId
    params["InstanceType"] = options.InstanceType
    var min, max int
    if options.MinCount == 0 && options.MaxCount == 0 {
        min = 1
        max = 1
    } else if options.MaxCount == 0 {
        min = options.MinCount
        max = min
    } else {
        min = options.MinCount
        max = options.MaxCount
    }
    params["MinCount"] = strconv.Itoa(min)
    params["MaxCount"] = strconv.Itoa(max)
    for i, name := range options.SecurityGroups {
        params["SecurityGroup."+strconv.Itoa(i+1)] = name
    }
    params["ClientToken"] = strconv.Itoa64(time.Nanoseconds()) // XXX Fix this.

    if options.KernelId != "" {
        params["KernelId"] = options.KernelId
    }
    if options.RamdiskId != "" {
        params["RamdiskId"] = options.RamdiskId
    }
    if options.UserData != nil {
        userData := make([]byte, b64.EncodedLen(len(options.UserData)))
        b64.Encode(userData, options.UserData)
        params["UserData"] = string(userData)
    }
    if options.AvailZone != "" {
        params["Placement.AvailabilityZone"] = options.AvailZone
    }
    if options.GroupName != "" {
        params["Placement.GroupName"] = options.GroupName
    }
    if options.Monitoring {
        params["Monitoring.Enabled"] = "true"
    }
    if options.SubnetId != "" {
        params["SubnetId"] = options.SubnetId
    }
    if options.DisableAPITermination {
        params["DisableApiTermination"] = "true"
    }
    if options.ShutdownBehavior != "" {
        params["InstanceInitiatedShutdownBehavior"] = options.ShutdownBehavior
    }
    if options.PrivateIPAddress != "" {
        params["PrivateIpAddress"] = options.PrivateIPAddress
    }

    resp = &RunInstancesResp{}
    err = ec2.query(params, resp)
    if err != nil {
        return nil, err
    }
    return
}

// ----------------------------------------------------------------------------
// TerminateInstances (http://goo.gl/3BKHj)

type TerminateInstancesResp struct {
    RequestId    string
    StateChanges []InstanceStateChange "InstancesSet>Item"
}

type InstanceState struct {
    Code int // Watch out, bits 8-15 have unpublished meaning.
    Name string
}

type InstanceStateChange struct {
    InstanceId    string
    CurrentState  InstanceState
    PreviousState InstanceState
}

func (ec2 *EC2) TerminateInstances(instanceIds []string) (resp *TerminateInstancesResp, err os.Error) {
    params := makeParams("TerminateInstances")
    for i, id := range instanceIds {
        params["InstanceId."+strconv.Itoa(i+1)] = id
    }
    resp = &TerminateInstancesResp{}
    err = ec2.query(params, resp)
    if err != nil {
        return nil, err
    }
    return
}

// ----------------------------------------------------------------------------
// DescribeInstances (http://goo.gl/mLbmw)

type DescribeInstancesResp struct {
    RequestId    string
    Reservations []Reservation "ReservationSet>Item"
}

type Reservation struct {
    ReservationId  string
    OwnerId        string
    RequesterId    string
    SecurityGroups []string   "GroupSet>Item>GroupId"
    Instances      []Instance "InstancesSet>Item"
}

func (ec2 *EC2) DescribeInstances(instanceIds []string, filter *Filter) (resp *DescribeInstancesResp, err os.Error) {
    params := makeParams("DescribeInstances")
    for i, id := range instanceIds {
        params["InstanceId."+strconv.Itoa(i+1)] = id
    }
    filter.addParams(params)

    resp = &DescribeInstancesResp{}
    err = ec2.query(params, resp)
    if err != nil {
        return nil, err
    }
    return
}

// ----------------------------------------------------------------------------
// CreateSecurityGroup (http://goo.gl/Eo7Yl)

type SimpleResp struct {
    RequestId string
}

func (ec2 *EC2) CreateSecurityGroup(name, description string) (resp *SimpleResp, err os.Error) {
    params := makeParams("CreateSecurityGroup")
    params["GroupName"] = name
    params["GroupDescription"] = description

    resp = &SimpleResp{}
    err = ec2.query(params, resp)
    if err != nil {
        return nil, err
    }
    return resp, nil
}

// ----------------------------------------------------------------------------
// DescribeSecurityGroups (http://goo.gl/k12Uy)

type DescribeSecurityGroupsResp struct {
    RequestId      string
    SecurityGroups []SecurityGroup "SecurityGroupInfo>Item"
}

type SecurityGroup struct {
    OwnerId       string
    GroupName     string         "GroupName>"
    GroupDescr    string         "GroupDescription>"
    IPPermissions []IPPermission "IPPermissions>Item"
}

type IPPermission struct {
    Protocol     string "IPProtocol>"
    FromPort     int
    ToPort       int
    SourceIPs    []string          "IPRanges>Item>CIDRIP"
    SourceGroups []SecurityGroupId "Groups>Item"
}

type SecurityGroupId struct {
    OwnerId   string "UserId>"
    GroupName string
}

func (ec2 *EC2) DescribeSecurityGroups(names []string, filter *Filter) (resp *DescribeSecurityGroupsResp, err os.Error) {
    params := makeParams("DescribeSecurityGroups")
    for i, name := range names {
        params["GroupName."+strconv.Itoa(i+1)] = name
    }
    filter.addParams(params)

    resp = &DescribeSecurityGroupsResp{}
    err = ec2.query(params, resp)
    if err != nil {
        return nil, err
    }
    return resp, nil
}

// ----------------------------------------------------------------------------
// DeleteSecurityGroup (http://goo.gl/QJJDO)

func (ec2 *EC2) DeleteSecurityGroup(name string) (resp *SimpleResp, err os.Error) {
    params := makeParams("DeleteSecurityGroup")
    params["GroupName"] = name

    resp = &SimpleResp{}
    err = ec2.query(params, resp)
    if err != nil {
        return nil, err
    }
    return resp, nil
}


// ----------------------------------------------------------------------------
// AuthorizeSecurityGroupIngress (http://goo.gl/u2sDJ)

func (ec2 *EC2) AuthorizeSecurityGroup(groupName string, perms []IPPermission) (resp *SimpleResp, err os.Error) {
    params := makeParams("AuthorizeSecurityGroupIngress")
    params["GroupName"] = groupName

    for i, perm := range perms {
        prefix := "IpPermissions." + strconv.Itoa(i+1)
        params[prefix+".IpProtocol"] = perm.Protocol
        params[prefix+".FromPort"] = strconv.Itoa(perm.FromPort)
        params[prefix+".ToPort"] = strconv.Itoa(perm.ToPort)
        for j, ip := range perm.SourceIPs {
            params[prefix+".IpRanges."+strconv.Itoa(j+1)+".CidrIp"] = ip
        }
        for j, g := range perm.SourceGroups {
            subprefix := prefix + ".Groups." + strconv.Itoa(j+1)
            params[subprefix+".UserId"] = g.OwnerId
            params[subprefix+".GroupName"] = g.GroupName
        }
    }

    resp = &SimpleResp{}
    err = ec2.query(params, resp)
    if err != nil {
        return nil, err
    }
    return resp, nil
}
