package goamz_test

import (
    .   "gocheck"
    "goamz"
)

var _ = Suite(&EC2Suite{})
var _ = Suite(&EC2IntSuite{})

type EC2Suite struct {
    HTTPSuite
    ec2 *goamz.EC2
}

type EC2IntSuite struct {
    IntSuite
    ec2 *goamz.EC2
}

func (s *EC2IntSuite) SetUpSuite(c *C) {
    s.IntSuite.SetUpSuite(c)
    s.ec2 = goamz.NewEC2(s.auth, goamz.USEast)
}

func (s *EC2Suite) SetUpSuite(c *C) {
    s.HTTPSuite.SetUpSuite(c)
    auth := goamz.Auth{"abc", "123"}
    s.ec2 = goamz.NewEC2(auth, goamz.Region{EC2Endpoint: fakeEndpoint})
}

func (s *EC2Suite) TestRunInstancesErrorDump(c *C) {
    s.FakeResponse(400, nil, ErrorDump)

    options := goamz.RunInstances{
        ImageId:      "ami-a6f504cf", // Ubuntu Maverick, i386, instance store
        InstanceType: "t1.micro",     // Doesn't work with micro, results in 400.
    }

    msg := "AMIs with an instance-store root device are not supported for the instance type 't1.micro'."

    resp, err := s.ec2.RunInstances(&options)

    s.WaitRequest()

    c.Assert(resp, IsNil)
    c.Assert(err, NotNil)
    c.Assert(err.String(), Equals, msg)

    ec2err, ok := err.(*goamz.EC2Error)
    c.Assert(ok, Equals, true)
    c.Assert(ec2err.StatusCode, Equals, 400)
    c.Assert(ec2err.Code, Equals, "UnsupportedOperation")
    c.Assert(ec2err.Message, Equals, msg)
    c.Assert(ec2err.RequestId, Equals, "0503f4e9-bbd6-483c-b54f-c4ae9f3b30f4")
}

func (s *EC2Suite) TestRunInstancesErrorWithoutXML(c *C) {
    s.FakeResponse(500, nil, "")
    options := goamz.RunInstances{ImageId: "image-id"}

    resp, err := s.ec2.RunInstances(&options)

    s.WaitRequest()

    c.Assert(resp, IsNil)
    c.Assert(err, Matches, "500 Internal Server Error")

    ec2err, ok := err.(*goamz.EC2Error)
    c.Assert(ok, Equals, true)
    c.Assert(ec2err.StatusCode, Equals, 500)
    c.Assert(ec2err.Code, Equals, "")
    c.Assert(ec2err.Message, Equals, "500 Internal Server Error")
    c.Assert(ec2err.RequestId, Equals, "")
}

func (s *EC2Suite) TestRunInstancesExample(c *C) {
    s.FakeResponse(200, nil, RunInstancesExample)

    options := goamz.RunInstances{
        ImageId:               "image-id",
        InstanceType:          "inst-type",
        SecurityGroups:        []string{"g1", "g2"},
        UserData:              []byte("1234"),
        KernelId:              "kernel-id",
        RamdiskId:             "ramdisk-id",
        AvailZone:             "zone",
        GroupName:             "group",
        Monitoring:            true,
        SubnetId:              "subnet-id",
        DisableAPITermination: true,
        ShutdownBehavior:      "terminate",
        PrivateIPAddress:      "10.0.0.25",
    }
    resp, err := s.ec2.RunInstances(&options)

    req := s.WaitRequest()
    c.Assert(req.Form["Action"], Equals, []string{"RunInstances"})
    c.Assert(req.Form["ImageId"], Equals, []string{"image-id"})
    c.Assert(req.Form["MinCount"], Equals, []string{"1"})
    c.Assert(req.Form["MaxCount"], Equals, []string{"1"})
    c.Assert(req.Form["InstanceType"], Equals, []string{"inst-type"})
    c.Assert(req.Form["SecurityGroup.1"], Equals, []string{"g1"})
    c.Assert(req.Form["SecurityGroup.2"], Equals, []string{"g2"})
    c.Assert(req.Form["UserData"], Equals, []string{"MTIzNA=="})
    c.Assert(req.Form["KernelId"], Equals, []string{"kernel-id"})
    c.Assert(req.Form["RamdiskId"], Equals, []string{"ramdisk-id"})
    c.Assert(req.Form["Placement.AvailabilityZone"], Equals, []string{"zone"})
    c.Assert(req.Form["Placement.GroupName"], Equals, []string{"group"})
    c.Assert(req.Form["Monitoring.Enabled"], Equals, []string{"true"})
    c.Assert(req.Form["SubnetId"], Equals, []string{"subnet-id"})
    c.Assert(req.Form["DisableApiTermination"], Equals, []string{"true"})
    c.Assert(req.Form["InstanceInitiatedShutdownBehavior"], Equals, []string{"terminate"})
    c.Assert(req.Form["PrivateIpAddress"], Equals, []string{"10.0.0.25"})

    c.Assert(err, IsNil)
    c.Assert(resp.RequestId, Equals, "59dbff89-35bd-4eac-99ed-be587EXAMPLE")
    c.Assert(resp.ReservationId, Equals, "r-47a5402e")
    c.Assert(resp.OwnerId, Equals, "999988887777")
    c.Assert(resp.SecurityGroups, Equals, []string{"default"})
    c.Assert(len(resp.Instances), Equals, 3)

    i0 := resp.Instances[0]
    c.Assert(i0.InstanceId, Equals, "i-2ba64342")
    c.Assert(i0.InstanceType, Equals, "m1.small")
    c.Assert(i0.ImageId, Equals, "ami-60a54009")
    c.Assert(i0.Monitoring, Equals, "enabled")
    c.Assert(i0.KeyName, Equals, "example-key-name")
    c.Assert(i0.AMILaunchIndex, Equals, 0)
    c.Assert(i0.VirtType, Equals, "paravirtual")
    c.Assert(i0.Hypervisor, Equals, "xen")

    i1 := resp.Instances[1]
    c.Assert(i1.InstanceId, Equals, "i-2bc64242")
    c.Assert(i1.InstanceType, Equals, "m1.small")
    c.Assert(i1.ImageId, Equals, "ami-60a54009")
    c.Assert(i1.Monitoring, Equals, "enabled")
    c.Assert(i1.KeyName, Equals, "example-key-name")
    c.Assert(i1.AMILaunchIndex, Equals, 1)
    c.Assert(i1.VirtType, Equals, "paravirtual")
    c.Assert(i1.Hypervisor, Equals, "xen")

    i2 := resp.Instances[2]
    c.Assert(i2.InstanceId, Equals, "i-2be64332")
    c.Assert(i2.InstanceType, Equals, "m1.small")
    c.Assert(i2.ImageId, Equals, "ami-60a54009")
    c.Assert(i2.Monitoring, Equals, "enabled")
    c.Assert(i2.KeyName, Equals, "example-key-name")
    c.Assert(i2.AMILaunchIndex, Equals, 2)
    c.Assert(i2.VirtType, Equals, "paravirtual")
    c.Assert(i2.Hypervisor, Equals, "xen")
}

func (s *EC2Suite) TestTerminateInstancesExample(c *C) {
    s.FakeResponse(200, nil, TerminateInstancesExample)

    resp, err := s.ec2.TerminateInstances([]string{"i-1", "i-2"})

    req := s.WaitRequest()
    c.Assert(req.Form["Action"], Equals, []string{"TerminateInstances"})
    c.Assert(req.Form["InstanceId.1"], Equals, []string{"i-1"})
    c.Assert(req.Form["InstanceId.2"], Equals, []string{"i-2"})
    c.Assert(req.Form["UserData"], Equals, []string{})
    c.Assert(req.Form["KernelId"], Equals, []string{})
    c.Assert(req.Form["RamdiskId"], Equals, []string{})
    c.Assert(req.Form["Placement.AvailabilityZone"], Equals, []string{})
    c.Assert(req.Form["Placement.GroupName"], Equals, []string{})
    c.Assert(req.Form["Monitoring.Enabled"], Equals, []string{})
    c.Assert(req.Form["SubnetId"], Equals, []string{})
    c.Assert(req.Form["DisableApiTermination"], Equals, []string{})
    c.Assert(req.Form["InstanceInitiatedShutdownBehavior"], Equals, []string{})
    c.Assert(req.Form["PrivateIpAddress"], Equals, []string{})

    c.Assert(err, IsNil)
    c.Assert(resp.RequestId, Equals, "59dbff89-35bd-4eac-99ed-be587EXAMPLE")
    c.Assert(len(resp.StateChanges), Equals, 1)
    c.Assert(resp.StateChanges[0].InstanceId, Equals, "i-3ea74257")
    c.Assert(resp.StateChanges[0].CurrentState.Code, Equals, 32)
    c.Assert(resp.StateChanges[0].CurrentState.Name, Equals, "shutting-down")
    c.Assert(resp.StateChanges[0].PreviousState.Code, Equals, 16)
    c.Assert(resp.StateChanges[0].PreviousState.Name, Equals, "running")
}

func (s *EC2Suite) TestDescribeInstancesExample1(c *C) {
    s.FakeResponse(200, nil, DescribeInstancesExample1)

    filter := goamz.NewFilter()
    filter.Add("key1", "value1")
    filter.Add("key2", "value2", "value3")

    resp, err := s.ec2.DescribeInstances([]string{"i-1", "i-2"}, nil)

    req := s.WaitRequest()
    c.Assert(req.Form["Action"], Equals, []string{"DescribeInstances"})
    c.Assert(req.Form["InstanceId.1"], Equals, []string{"i-1"})
    c.Assert(req.Form["InstanceId.2"], Equals, []string{"i-2"})

    c.Assert(err, IsNil)
    c.Assert(resp.RequestId, Equals, "98e3c9a4-848c-4d6d-8e8a-b1bdEXAMPLE")
    c.Assert(len(resp.Reservations), Equals, 2)

    r0 := resp.Reservations[0]
    c.Assert(r0.ReservationId, Equals, "r-b27e30d9")
    c.Assert(r0.OwnerId, Equals, "999988887777")
    c.Assert(r0.RequesterId, Equals, "854251627541")
    c.Assert(r0.SecurityGroups, Equals, []string{"default"})
    c.Assert(len(r0.Instances), Equals, 1)

    r0i := r0.Instances[0]
    c.Assert(r0i.InstanceId, Equals, "i-c5cd56af")
    c.Assert(r0i.PrivateDNSName, Equals, "domU-12-31-39-10-56-34.compute-1.internal")
    c.Assert(r0i.DNSName, Equals, "ec2-174-129-165-232.compute-1.amazonaws.com")
    c.Assert(r0i.AvailZone, Equals, "us-east-1b")
}

func (s *EC2Suite) TestDescribeInstancesExample2(c *C) {
    s.FakeResponse(200, nil, DescribeInstancesExample2)

    filter := goamz.NewFilter()
    filter.Add("key1", "value1")
    filter.Add("key2", "value2", "value3")

    resp, err := s.ec2.DescribeInstances([]string{"i-1", "i-2"}, filter)

    req := s.WaitRequest()
    c.Assert(req.Form["Action"], Equals, []string{"DescribeInstances"})
    c.Assert(req.Form["InstanceId.1"], Equals, []string{"i-1"})
    c.Assert(req.Form["InstanceId.2"], Equals, []string{"i-2"})
    c.Assert(req.Form["Filter.1.Name"], Equals, []string{"key1"})
    c.Assert(req.Form["Filter.1.Value.1"], Equals, []string{"value1"})
    c.Assert(req.Form["Filter.1.Value.2"], Equals, []string{})
    c.Assert(req.Form["Filter.2.Name"], Equals, []string{"key2"})
    c.Assert(req.Form["Filter.2.Value.1"], Equals, []string{"value2"})
    c.Assert(req.Form["Filter.2.Value.2"], Equals, []string{"value3"})

    c.Assert(err, IsNil)
    c.Assert(resp.RequestId, Equals, "59dbff89-35bd-4eac-99ed-be587EXAMPLE")
    c.Assert(len(resp.Reservations), Equals, 1)
}

func (s *EC2Suite) TestCreateSecurityGroupExample(c *C) {
    s.FakeResponse(200, nil, CreateSecurityGroupExample)

    resp, err := s.ec2.CreateSecurityGroup("websrv", "Web Servers")

    req := s.WaitRequest()
    c.Assert(req.Form["Action"], Equals, []string{"CreateSecurityGroup"})
    c.Assert(req.Form["GroupName"], Equals, []string{"websrv"})
    c.Assert(req.Form["GroupDescription"], Equals, []string{"Web Servers"})

    c.Assert(err, IsNil)
    c.Assert(resp.RequestId, Equals, "59dbff89-35bd-4eac-99ed-be587EXAMPLE")
}

func (s *EC2Suite) TestDescribeSecurityGroupsExample(c *C) {
    s.FakeResponse(200, nil, DescribeSecurityGroupsExample)

    resp, err := s.ec2.DescribeSecurityGroups([]string{"WebServers", "RangedPortsBySource"}, nil)

    req := s.WaitRequest()
    c.Assert(req.Form["Action"], Equals, []string{"DescribeSecurityGroups"})
    c.Assert(req.Form["GroupName.1"], Equals, []string{"WebServers"})
    c.Assert(req.Form["GroupName.2"], Equals, []string{"RangedPortsBySource"})

    c.Assert(err, IsNil)
    c.Assert(resp.RequestId, Equals, "59dbff89-35bd-4eac-99ed-be587EXAMPLE")
    c.Assert(len(resp.SecurityGroups), Equals, 2)

    g0 := resp.SecurityGroups[0]
    c.Assert(g0.OwnerId, Equals, "999988887777")
    c.Assert(g0.GroupName, Equals, "WebServers")
    c.Assert(g0.GroupDescr, Equals, "Web Servers")
    c.Assert(len(g0.IPPermissions), Equals, 1)

    g0ipp := g0.IPPermissions[0]
    c.Assert(g0ipp.Protocol, Equals, "tcp")
    c.Assert(g0ipp.FromPort, Equals, 80)
    c.Assert(g0ipp.ToPort, Equals, 80)
    c.Assert(g0ipp.SourceIPs, Equals, []string{"0.0.0.0/0"})

    g1 := resp.SecurityGroups[1]
    c.Assert(g1.OwnerId, Equals, "999988887777")
    c.Assert(g1.GroupName, Equals, "RangedPortsBySource")
    c.Assert(g1.GroupDescr, Equals, "Group A")
    c.Assert(len(g1.IPPermissions), Equals, 1)

    g1ipp := g1.IPPermissions[0]
    c.Assert(g1ipp.Protocol, Equals, "tcp")
    c.Assert(g1ipp.FromPort, Equals, 6000)
    c.Assert(g1ipp.ToPort, Equals, 7000)
    c.Assert(g1ipp.SourceIPs, Equals, []string{})
}

func (s *EC2Suite) TestDescribeSecurityGroupsExampleWithFilter(c *C) {
    s.FakeResponse(200, nil, DescribeSecurityGroupsExample)

    filter := goamz.NewFilter()
    filter.Add("ip-permission.protocol", "tcp")
    filter.Add("ip-permission.from-port", "22")
    filter.Add("ip-permission.to-port", "22")
    filter.Add("ip-permission.group-name", "app_server_group", "database_group")

    _, err := s.ec2.DescribeSecurityGroups(nil, filter)

    req := s.WaitRequest()
    c.Assert(req.Form["Action"], Equals, []string{"DescribeSecurityGroups"})
    c.Assert(req.Form["Filter.1.Name"], Equals, []string{"ip-permission.from-port"})
    c.Assert(req.Form["Filter.1.Value.1"], Equals, []string{"22"})
    c.Assert(req.Form["Filter.2.Name"], Equals, []string{"ip-permission.group-name"})
    c.Assert(req.Form["Filter.2.Value.1"], Equals, []string{"app_server_group"})
    c.Assert(req.Form["Filter.2.Value.2"], Equals, []string{"database_group"})
    c.Assert(req.Form["Filter.3.Name"], Equals, []string{"ip-permission.protocol"})
    c.Assert(req.Form["Filter.3.Value.1"], Equals, []string{"tcp"})
    c.Assert(req.Form["Filter.4.Name"], Equals, []string{"ip-permission.to-port"})
    c.Assert(req.Form["Filter.4.Value.1"], Equals, []string{"22"})

    c.Assert(err, IsNil)
}

func (s *EC2Suite) TestDescribeSecurityGroupsDumpWithGroup(c *C) {
    s.FakeResponse(200, nil, DescribeSecurityGroupsDump)

    resp, err := s.ec2.DescribeSecurityGroups(nil, nil)

    req := s.WaitRequest()
    c.Assert(req.Form["Action"], Equals, []string{"DescribeSecurityGroups"})

    c.Assert(err, IsNil)
    c.Assert(len(resp.SecurityGroups), Equals, 1)
    c.Assert(len(resp.SecurityGroups[0].IPPermissions), Equals, 2)

    ipp0 := resp.SecurityGroups[0].IPPermissions[0]
    c.Assert(len(ipp0.SourceIPs), Equals, 0)
    c.Assert(len(ipp0.SourceGroups), Equals, 1)
    c.Assert(ipp0.SourceGroups[0].OwnerId, Equals, "12345")
    c.Assert(ipp0.SourceGroups[0].GroupName, Equals, "default")

    ipp1 := resp.SecurityGroups[0].IPPermissions[1]
    c.Assert(len(ipp0.SourceIPs), Equals, 0)
    c.Assert(len(ipp0.SourceGroups), Equals, 1)
    c.Assert(ipp1.SourceGroups[0].OwnerId, Equals, "12345")
    c.Assert(ipp1.SourceGroups[0].GroupName, Equals, "default")
}

func (s *EC2Suite) TestDeleteSecurityGroupExample(c *C) {
    s.FakeResponse(200, nil, DeleteSecurityGroupExample)

    resp, err := s.ec2.DeleteSecurityGroup("websrv")

    req := s.WaitRequest()
    c.Assert(req.Form["Action"], Equals, []string{"DeleteSecurityGroup"})
    c.Assert(req.Form["GroupName"], Equals, []string{"websrv"})

    c.Assert(err, IsNil)
    c.Assert(resp.RequestId, Equals, "59dbff89-35bd-4eac-99ed-be587EXAMPLE")
}

func (s *EC2Suite) TestAuthorizeSecurityGroupExample1(c *C) {
    s.FakeResponse(200, nil, AuthorizeSecurityGroupIngressExample)

    perms := []goamz.IPPermission{{
        Protocol:  "tcp",
        FromPort:  80,
        ToPort:    80,
        SourceIPs: []string{"205.192.0.0/16", "205.159.0.0/16"},
    }}
    resp, err := s.ec2.AuthorizeSecurityGroup("websrv", perms)

    req := s.WaitRequest()

    c.Assert(req.Form["Action"], Equals, []string{"AuthorizeSecurityGroupIngress"})
    c.Assert(req.Form["GroupName"], Equals, []string{"websrv"})
    c.Assert(req.Form["IpPermissions.1.IpProtocol"], Equals, []string{"tcp"})
    c.Assert(req.Form["IpPermissions.1.FromPort"], Equals, []string{"80"})
    c.Assert(req.Form["IpPermissions.1.ToPort"], Equals, []string{"80"})
    c.Assert(req.Form["IpPermissions.1.IpRanges.1.CidrIp"], Equals, []string{"205.192.0.0/16"})
    c.Assert(req.Form["IpPermissions.1.IpRanges.2.CidrIp"], Equals, []string{"205.159.0.0/16"})

    c.Assert(err, IsNil)
    c.Assert(resp.RequestId, Equals, "59dbff89-35bd-4eac-99ed-be587EXAMPLE")
}

func (s *EC2Suite) TestAuthorizeSecurityGroupExample2(c *C) {
    s.FakeResponse(200, nil, AuthorizeSecurityGroupIngressExample)

    perms := []goamz.IPPermission{{
        Protocol:     "tcp",
        FromPort:     80,
        ToPort:       81,
        SourceGroups: []goamz.SecurityGroupId{{"999988887777", "OtherAccountGroup"}},
    }}
    resp, err := s.ec2.AuthorizeSecurityGroup("websrv", perms)

    req := s.WaitRequest()

    c.Assert(req.Form["Action"], Equals, []string{"AuthorizeSecurityGroupIngress"})
    c.Assert(req.Form["GroupName"], Equals, []string{"websrv"})
    c.Assert(req.Form["IpPermissions.1.IpProtocol"], Equals, []string{"tcp"})
    c.Assert(req.Form["IpPermissions.1.FromPort"], Equals, []string{"80"})
    c.Assert(req.Form["IpPermissions.1.ToPort"], Equals, []string{"81"})
    c.Assert(req.Form["IpPermissions.1.Groups.1.UserId"], Equals, []string{"999988887777"})
    c.Assert(req.Form["IpPermissions.1.Groups.1.GroupName"], Equals, []string{"OtherAccountGroup"})

    c.Assert(err, IsNil)
    c.Assert(resp.RequestId, Equals, "59dbff89-35bd-4eac-99ed-be587EXAMPLE")
}
