package goamz_test

import (
    .   "gocheck"
    "goamz"
)

// Suite cost per run: 0.02 USD

var imageId = "ami-ccf405a5" // Ubuntu Maverick, i386, EBS store

// Cost: 0.00 USD
func (s *EC2IntSuite) TestRunInstancesError(c *C) {
    options := goamz.RunInstances{
        ImageId:      "ami-a6f504cf", // Ubuntu Maverick, i386, instance store
        InstanceType: "t1.micro",     // Doesn't work with micro, results in 400.
    }

    msg := "AMIs with an instance-store root device are not supported for the instance type 't1.micro'."

    resp, err := s.ec2.RunInstances(&options)

    c.Assert(resp, IsNil)
    c.Assert(err, NotNil)
    c.Assert(err.String(), Equals, msg)

    ec2err, ok := err.(*goamz.EC2Error)
    c.Assert(ok, Equals, true)
    c.Assert(ec2err.StatusCode, Equals, 400)
    c.Assert(ec2err.Code, Equals, "UnsupportedOperation")
    c.Assert(ec2err.Message, Equals, msg)
    c.Assert(ec2err.RequestId, Matches, ".+")
}

// Cost: 0.02 USD
func (s *EC2IntSuite) TestRunAndTerminate(c *C) {
    options := goamz.RunInstances{
        ImageId:      imageId,
        InstanceType: "t1.micro",
    }
    resp1, err := s.ec2.RunInstances(&options)
    c.Assert(err, IsNil)
    c.Check(resp1.ReservationId, Matches, "r-[0-9a-f]*")
    c.Check(resp1.OwnerId, Matches, "[0-9]+")
    c.Check(len(resp1.Instances), Equals, 1)
    c.Check(resp1.Instances[0].InstanceType, Equals, "t1.micro")

    instId := resp1.Instances[0].InstanceId

    resp2, err := s.ec2.DescribeInstances([]string{instId}, nil)
    c.Check(err, IsNil)
    if c.Check(len(resp2.Reservations), Equals, 1) && c.Check(len(resp2.Reservations[0].Instances), Equals, 1) {
        inst := resp2.Reservations[0].Instances[0]
        c.Check(inst.InstanceId, Equals, instId)
    }

    resp3, err := s.ec2.TerminateInstances([]string{instId})
    c.Assert(err, IsNil)
    c.Check(len(resp3.StateChanges), Equals, 1)
    c.Check(resp3.StateChanges[0].InstanceId, Equals, instId)
    c.Check(resp3.StateChanges[0].CurrentState.Name, Equals, "shutting-down")
    c.Check(resp3.StateChanges[0].CurrentState.Code, Equals, 32)
}

// Cost: 0.00 USD
func (s *EC2IntSuite) TestSecurityGroups(c *C) {
    name := "goamz-test"
    descr := "goamz security group for tests"

    // Clean it up, if a previous test left it around and avoid leaving it around.
    s.ec2.DeleteSecurityGroup(name)
    defer s.ec2.DeleteSecurityGroup(name)

    resp1, err := s.ec2.CreateSecurityGroup(name, descr)
    c.Assert(err, IsNil)
    c.Assert(resp1.RequestId, Matches, ".+")

    resp1, err = s.ec2.CreateSecurityGroup(name, descr)
    ec2err, _ := err.(*goamz.EC2Error)
    c.Assert(resp1, IsNil)
    c.Assert(ec2err, NotNil)
    c.Assert(ec2err.Code, Equals, "InvalidGroup.Duplicate")

    perms := []goamz.IPPermission{{
        Protocol:  "tcp",
        FromPort:  0,
        ToPort:    1024,
        SourceIPs: []string{"127.0.0.1/24"},
    }}

    resp1, err = s.ec2.AuthorizeSecurityGroup(name, perms)
    c.Assert(err, IsNil)
    c.Assert(resp1.RequestId, Matches, ".+")

    resp2, err := s.ec2.DescribeSecurityGroups([]string{name}, nil)
    c.Assert(err, IsNil)
    c.Assert(resp2.RequestId, Matches, ".+")
    c.Assert(len(resp2.SecurityGroups), Equals, 1)

    g0 := resp2.SecurityGroups[0]
    c.Assert(g0.GroupName, Equals, name)
    c.Assert(g0.GroupDescr, Equals, descr)
    c.Assert(len(g0.IPPermissions), Equals, 1)
    c.Assert(g0.IPPermissions[0].Protocol, Equals, "tcp")
    c.Assert(g0.IPPermissions[0].FromPort, Equals, 0)
    c.Assert(g0.IPPermissions[0].ToPort, Equals, 1024)
    c.Assert(g0.IPPermissions[0].SourceIPs, Equals, []string{"127.0.0.1/24"})

    resp1, err = s.ec2.DeleteSecurityGroup(name)
    c.Assert(err, IsNil)
    c.Assert(resp1.RequestId, Matches, ".+")
}
