package goamz

import (
    "crypto/hmac"
    "encoding/base64"
    "sort"
    "strings"
)

// ----------------------------------------------------------------------------
// EC2 signing (http://goo.gl/fQmAN)

var b64 = base64.StdEncoding
var hex = "0123456789ABCDEF"
var unreserved = make([]bool, 128)

func init() {
    // RFC3986
    u := "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz01234567890-_.~"
    for _, c := range u {
        unreserved[c] = true
    }
}

func signEC2(auth Auth, method, path string, params map[string]string, host string) {
    params["AWSAccessKeyId"] = auth.AccessKey
    params["SignatureVersion"] = "2"
    params["SignatureMethod"] = "HmacSHA256"

    var sarray sort.StringArray
    for k, v := range params {
        sarray = append(sarray, sencode(k)+"="+sencode(v))
    }
    sort.Sort(sarray)
    joined := strings.Join(sarray, "&")
    payload := method + "\n" + host + "\n" + path + "\n" + joined
    hash := hmac.NewSHA256([]byte(auth.SecretKey))
    hash.Write([]byte(payload))
    signature := make([]byte, b64.EncodedLen(hash.Size()))
    b64.Encode(signature, hash.Sum())

    params["Signature"] = string(signature)
}

func sencode(s string) string {
    encode := false
    for i := 0; i != len(s); i++ {
        c := s[i]
        if c > 127 || !unreserved[c] {
            encode = true
            break
        }
    }
    if !encode {
        return s
    }
    e := make([]byte, len(s)*3)
    ei := 0
    for i := 0; i != len(s); i++ {
        c := s[i]
        if c > 127 || !unreserved[c] {
            e[ei] = '%'
            e[ei+1] = hex[c>>4]
            e[ei+2] = hex[c&0xF]
            ei += 3
        } else {
            e[ei] = c
            ei += 1
        }
    }
    return string(e[:ei])
}


// ----------------------------------------------------------------------------
// S3 signing (http://goo.gl/G1LrK)

var s3ParamsToSign = map[string]bool{
    "acl":                          true,
    "location":                     true,
    "logging":                      true,
    "notification":                 true,
    "partNumber":                   true,
    "policy":                       true,
    "requestPayment":               true,
    "torrent":                      true,
    "uploadId":                     true,
    "uploads":                      true,
    "versionId":                    true,
    "versioning":                   true,
    "versions":                     true,
    "response-content-type":        true,
    "response-content-language":    true,
    "response-expires":             true,
    "response-cache-control":       true,
    "response-content-disposition": true,
    "response-content-encoding":    true,
}

func signS3(auth Auth, method, path string, params, headers map[string]string) {
    var host, md5, ctype, date, xamz string
    var xamzDate bool
    var sarray sort.StringArray
    for k, v := range headers {
        k = strings.ToLower(k)
        switch k {
        case "host":
            host = v
        case "content-md5":
            md5 = v
        case "content-type":
            ctype = v
        case "date":
            if !xamzDate {
                date = v
            }
        default:
            if strings.HasPrefix(k, "x-amz-") {
                sarray = append(sarray, k+":"+v)
                if k == "x-amz-date" {
                    xamzDate = true
                    date = ""
                }
            }
        }
    }
    if len(sarray) > 0 {
        sort.Sort(sarray)
        xamz = strings.Join(sarray, "\n") + "\n"
    }

    colon := strings.Index(host, ":")
    if colon != -1 {
        host = host[:colon]
    }

    if strings.HasSuffix(host, ".amazonaws.com") {
        parts := strings.Split(host, ".", -1)
        // -3 also strips out .s3. (or .s3-us-west-1., etc)
        bucket := strings.Join(parts[:len(parts)-3], ".")
        if bucket != "" {
            path = "/" + bucket + path
        }
    } else {
        path = "/" + host + path
    }

    sarray = sarray[0:0]
    for k, v := range params {
        if s3ParamsToSign[k] {
            if v == "" {
                sarray = append(sarray, k)
            } else {
                // "When signing you do not encode these values."
                sarray = append(sarray, k+"="+v)
            }
        }
    }
    if len(sarray) > 0 {
        sort.Sort(sarray)
        path = path + "?" + strings.Join(sarray, "&")
    }

    payload := method + "\n" + md5 + "\n" + ctype + "\n" + date + "\n" + xamz + path
    //fmt.Printf("payload: %q\n", payload)
    hash := hmac.NewSHA1([]byte(auth.SecretKey))
    hash.Write([]byte(payload))
    signature := make([]byte, b64.EncodedLen(hash.Size()))
    b64.Encode(signature, hash.Sum())

    headers["Authorization"] = "AWS " + auth.AccessKey + ":" + string(signature)
}
