package goamz_test

import (
    "goamz"
    .   "gocheck"
)

var _ = Suite(&SignS3Suite{})
var _ = Suite(&SignEC2Suite{})

type SignEC2Suite struct{}
type SignS3Suite struct{}

// EC2 ReST authentication docs: http://goo.gl/fQmAN

var testEC2Auth = goamz.Auth{"user", "secret"}

func (s *SignEC2Suite) TestBasicSignature(c *C) {
    params := map[string]string{}
    goamz.SignEC2(testEC2Auth, "GET", "/path", params, "localhost")
    c.Assert(params["SignatureVersion"], Equals, "2")
    c.Assert(params["SignatureMethod"], Equals, "HmacSHA256")
    expected := "6lSe5QyXum0jMVc7cOUz32/52ZnL7N5RyKRk/09yiK4="
    c.Assert(params["Signature"], Equals, expected)
}

func (s *SignEC2Suite) TestParamSignature(c *C) {
    params := map[string]string{
        "param1": "value1",
        "param2": "value2",
        "param3": "value3",
    }
    goamz.SignEC2(testEC2Auth, "GET", "/path", params, "localhost")
    expected := "XWOR4+0lmK8bD8CGDGZ4kfuSPbb2JibLJiCl/OPu1oU="
    c.Assert(params["Signature"], Equals, expected)
}

func (s *SignEC2Suite) TestEscaping(c *C) {
    params := map[string]string{"Nonce": "+ +"}
    goamz.SignEC2(testEC2Auth, "GET", "/path", params, "localhost")
    expected := "bqffDELReIqwjg/W0DnsnVUmfLK4wXVLO4/LuG+1VFA="
    c.Assert(params["Signature"], Equals, expected)
}

func (s *SignEC2Suite) TestSignatureExample1(c *C) {
    params := map[string]string{
        "Timestamp": "2009-02-01T12:53:20+00:00",
        "Version":   "2007-11-07",
        "Action":    "ListDomains",
    }
    goamz.SignEC2(goamz.Auth{"access", "secret"}, "GET", "/", params, "sdb.amazonaws.com")
    expected := "okj96/5ucWBSc1uR2zXVfm6mDHtgfNv657rRtt/aunQ="
    c.Assert(params["Signature"], Equals, expected)
}


// S3 ReST authentication docs: http://goo.gl/G1LrK

var testS3Auth = goamz.Auth{"0PN5J17HBGZHT7JJ3X82", "uV3F3YluFJax1cknvbcGwgjvx4QpvB+leU8dUj2o"}

func (s *SignS3Suite) TestExampleObjectGet(c *C) {
    method := "GET"
    path := "/photos/puppy.jpg"
    headers := map[string]string{
        "Host": "johnsmith.s3.amazonaws.com",
        "Date": "Tue, 27 Mar 2007 19:36:42 +0000",
    }
    goamz.SignS3(testS3Auth, method, path, nil, headers)
    expected := "AWS 0PN5J17HBGZHT7JJ3X82:xXjDGYUmKxnwqr5KXNPGldn5LbA="
    c.Assert(headers["Authorization"], Equals, expected)
}

func (s *SignS3Suite) TestExampleObjectPut(c *C) {
    method := "PUT"
    path := "/photos/puppy.jpg"
    headers := map[string]string{
        "Host":           "johnsmith.s3.amazonaws.com",
        "Date":           "Tue, 27 Mar 2007 21:15:45 +0000",
        "Content-Type":   "image/jpeg",
        "Content-Length": "94328",
    }
    goamz.SignS3(testS3Auth, method, path, nil, headers)
    expected := "AWS 0PN5J17HBGZHT7JJ3X82:hcicpDDvL9SsO6AkvxqmIWkmOuQ="
    c.Assert(headers["Authorization"], Equals, expected)
}

func (s *SignS3Suite) TestExampleList(c *C) {
    method := "GET"
    path := "/"
    params := map[string]string{
        "prefix":   "photos",
        "max-keys": "50",
        "marker":   "puppy",
    }
    headers := map[string]string{
        "Host":       "johnsmith.s3.amazonaws.com",
        "Date":       "Tue, 27 Mar 2007 19:42:41 +0000",
        "User-Agent": "Mozilla/5.0",
    }
    goamz.SignS3(testS3Auth, method, path, params, headers)
    expected := "AWS 0PN5J17HBGZHT7JJ3X82:jsRt/rhG+Vtp88HrYL706QhE4w4="
    c.Assert(headers["Authorization"], Equals, expected)
}

func (s *SignS3Suite) TestExampleFetch(c *C) {
    method := "GET"
    path := "/"
    params := map[string]string{
        "acl": "",
    }
    headers := map[string]string{
        "Host": "johnsmith.s3.amazonaws.com",
        "Date": "Tue, 27 Mar 2007 19:44:46 +0000",
    }
    goamz.SignS3(testS3Auth, method, path, params, headers)
    expected := "AWS 0PN5J17HBGZHT7JJ3X82:thdUi9VAkzhkniLj96JIrOPGi0g="
    c.Assert(headers["Authorization"], Equals, expected)
}

func (s *SignS3Suite) TestExampleDelete(c *C) {
    method := "DELETE"
    path := "/johnsmith/photos/puppy.jpg"
    params := map[string]string{}
    headers := map[string]string{
        "Host":       "s3.amazonaws.com",
        "Date":       "Tue, 27 Mar 2007 21:20:27 +0000",
        "User-Agent": "dotnet",
        "x-amz-date": "Tue, 27 Mar 2007 21:20:26 +0000",
    }
    goamz.SignS3(testS3Auth, method, path, params, headers)
    expected := "AWS 0PN5J17HBGZHT7JJ3X82:k3nL7gH3+PadhTEVn5Ip83xlYzk="
    c.Assert(headers["Authorization"], Equals, expected)
}

func (s *SignS3Suite) TestExampleUpload(c *C) {
    method := "PUT"
    path := "/db-backup.dat.gz"
    params := map[string]string{}
    headers := map[string]string{
        "Host":                         "static.johnsmith.net:8080",
        "Date":                         "Tue, 27 Mar 2007 21:06:08 +0000",
        "User-Agent":                   "curl/7.15.5",
        "x-amz-acl":                    "public-read",
        "content-type":                 "application/x-download",
        "Content-MD5":                  "4gJE4saaMU4BqNR0kLY+lw==",
        "X-Amz-Meta-ReviewedBy":        "joe@johnsmith.net,jane@johnsmith.net",
        "X-Amz-Meta-FileChecksum":      "0x02661779",
        "X-Amz-Meta-ChecksumAlgorithm": "crc32",
        "Content-Disposition":          "attachment; filename=database.dat",
        "Content-Encoding":             "gzip",
        "Content-Length":               "5913339",
    }
    goamz.SignS3(testS3Auth, method, path, params, headers)
    expected := "AWS 0PN5J17HBGZHT7JJ3X82:C0FlOtU8Ylb9KDTpZqYkZPX91iI="
    c.Assert(headers["Authorization"], Equals, expected)
}

func (s *SignS3Suite) TestExampleListAllMyBuckets(c *C) {
    method := "GET"
    path := "/"
    headers := map[string]string{
        "Host": "s3.amazonaws.com",
        "Date": "Wed, 28 Mar 2007 01:29:59 +0000",
    }
    goamz.SignS3(testS3Auth, method, path, nil, headers)
    expected := "AWS 0PN5J17HBGZHT7JJ3X82:Db+gepJSUbZKwpx1FR0DLtEYoZA="
    c.Assert(headers["Authorization"], Equals, expected)
}

func (s *SignS3Suite) TestExampleUnicodeKeys(c *C) {
    method := "GET"
    path := "/dictionary/fran%C3%A7ais/pr%c3%a9f%c3%a8re"
    headers := map[string]string{
        "Host": "s3.amazonaws.com",
        "Date": "Wed, 28 Mar 2007 01:49:49 +0000",
    }
    goamz.SignS3(testS3Auth, method, path, nil, headers)
    expected := "AWS 0PN5J17HBGZHT7JJ3X82:dxhSBHoI6eVSPcXJqEghlUzZMnY="
    c.Assert(headers["Authorization"], Equals, expected)
}
